#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Настройки
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
import warnings  # Подавление Warning

for warn in [UserWarning, FutureWarning]:
    warnings.filterwarnings("ignore", category=warn)

import os  # Взаимодействие с файловой системой

from dataclasses import dataclass  # Класс данных

from colorama import init  # Цветной текст терминала

# Типы данных
from typing import List

# Персональные
from openav.modules.core.messages import Messages  # Сообщения

# ######################################################################################################################
# Константы
# ######################################################################################################################
PATH_TO_SAVE_MODELS: str = "./models"  # Путь к директории для сохранения моделей
PATH_TO_DATASET: str = "./dataset"  # Путь к директории набора данных
PATH_TO_INPUT_AUGMENTATION_DIRECTORY: str = "./data_aug"  # Путь к директории набора данных для обработки
PATH_TO_OUTPUT_AUGMENTATION_DIRECTORY: str = "./data_aug_processed"  # Путь к директории результирующих данных
# Путь к директории набора данных состоящего из фрагментов аудиовизуального сигнала (VAD)
PATH_TO_DATASET_VAD: str = "./dataset_vad"
# Путь к директории набора данных состоящего из фрагментов аудиовизуального сигнала (VOSK)
PATH_TO_DATASET_VOSK_SR: str = "./dataset_vosk"
IGNORE_DIRS: List[str] = []  # Директории не входящие в выборку
# Названия директорий для видео и аудио
DIR_VA_NAMES: List[str] = [
    "Video",
    "Audio",
]
# Расширения искомых файлов
EXT_SEARCH_FILES: List[str] = [
    "mov",
    "mp4",
    "webm",
    "wav",
]
CHUNK_SIZE: int = 1000000  # Размер загрузки файла из сети за 1 шаг

COLOR_INFO: str = "#1776D2"  # Цвет текста содержащего информацию (шестнадцатеричный код)
COLOR_SIMPLE: str = "#666"  # Цвет обычного текста (шестнадцатеричный код)
COLOR_ERR: str = "#FF0000"  # Цвет текста содержащего ошибку (шестнадцатеричный код)
COLOR_TRUE: str = "#008001"  # Цвет текста содержащего положительную информацию (шестнадцатеричный код)


# ######################################################################################################################
# Настройки
# ######################################################################################################################
@dataclass
class Settings(Messages):
    """Класс для настроек

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        init()  # Инициализация терминала с возможностью цветного текста

        self.__color_green: str = "\033[92m"  # Зеленый
        self.__color_red: str = "\033[91m"  # Красный
        self.__color_blue: str = "\033[94m"  # Синий
        self.__text_bold: str = "\033[1m"  # Жирный
        self.__clear_line: str = "\x1b[2K"  # Очистка линии в терминале
        self.__text_end: str = "\033[0m"  # Выход

        self.path_to_save_models: str = PATH_TO_SAVE_MODELS  # Путь к директории для сохранения моделей
        self.path_to_dataset: str = PATH_TO_DATASET  # Путь к директории набора данных
        self.path_to_input_augmentation_directory: str = (
            PATH_TO_INPUT_AUGMENTATION_DIRECTORY  # Путь к директории набора данных для обработки
        )
        self.path_to_output_augmentation_directory: str = (
            PATH_TO_OUTPUT_AUGMENTATION_DIRECTORY  # Путь к директории результирующих данных аугментации
        )
        # Путь к директории набора данных состоящего из фрагментов аудиовизуального сигнала (VAD)
        self.path_to_dataset_vad: str = PATH_TO_DATASET_VAD
        # Путь к директории набора данных состоящего из фрагментов аудиовизуального сигнала (VOSK)
        self.path_to_dataset_vosk_sr: str = PATH_TO_DATASET_VOSK_SR
        self.ignore_dirs: List[str] = IGNORE_DIRS  # Директории не входящие в выборку
        self.dir_va_names: List[str] = DIR_VA_NAMES  # Названия директорий для видео и аудио
        self.ext_search_files: List[str] = EXT_SEARCH_FILES  # Расширения искомых файлов
        self.chunk_size: int = CHUNK_SIZE  # Размер загрузки файла из сети за 1 шаг

    # ------------------------------------------------------------------------------------------------------------------
    # Свойства
    # ------------------------------------------------------------------------------------------------------------------

    @property
    def color_green(self) -> str:
        """Получение зеленого цвета текста в терминале

        Returns:
            str: Цвет текста в терминале
        """

        return self.__color_green

    @property
    def color_red(self) -> str:
        """Получение красного цвета текста в терминале

        Returns:
            str: Цвет текста в терминале
        """

        return self.__color_red

    @property
    def color_blue(self) -> str:
        """Получение синего цвета текста в терминале

        Returns:
            str: Цвет текста в терминале
        """

        return self.__color_blue

    @property
    def text_bold(self) -> str:
        """Получение жирного начертания текста в терминале

        Returns:
            str: Жирное начертание текста в терминале
        """

        return self.__text_bold

    @property
    def clear_line(self) -> str:
        """Получение очистки линии в терминале

        Returns:
            str: Очистка линии в терминале
        """

        return self.__clear_line

    @property
    def text_end(self) -> str:
        """Получение сброса оформления текста в терминале

        Returns:
            str: Сброс оформления текста в терминале
        """

        return self.__text_end

    @property
    def path_to_save_models(self) -> str:
        """Получение/установка пути к директории для сохранения моделей

        Args:
            (str): Путь

        Returns:
            str: Путь
        """

        return self.__path_to_save_models

    @path_to_save_models.setter
    def path_to_save_models(self, path: str):
        """Установка пути к директории для сохранения моделей"""

        try:
            # Проверка аргументов
            if type(path) is not str or not path:
                raise TypeError
        except TypeError:
            pass
        else:
            self.__path_to_save_models = os.path.normpath(path.strip())

    @property
    def path_to_dataset(self) -> str:
        """Получение/установка пути к директории набора данных

        Args:
            (str): Путь

        Returns:
            str: Путь
        """

        return self.__path_to_dataset

    @path_to_dataset.setter
    def path_to_dataset(self, path: str):
        """Установка пути к директории набора данных"""

        try:
            # Проверка аргументов
            if type(path) is not str or not path:
                raise TypeError
        except TypeError:
            pass
        else:
            self.__path_to_dataset = os.path.normpath(path.strip())

    @property
    def path_to_dataset_vad(self) -> str:
        """Получение/установка пути к директории набора данных состоящего из фрагментов аудиовизуального сигнала (VAD)

        Args:
            (str): Путь

        Returns:
            str: Путь
        """

        return self.__path_to_dataset_vad

    @path_to_dataset_vad.setter
    def path_to_dataset_vad(self, path: str):
        """Установка пути к директории набора данных состоящего из фрагментов аудиовизуального сигнала (VAD)"""

        try:
            # Проверка аргументов
            if type(path) is not str or not path:
                raise TypeError
        except TypeError:
            pass
        else:
            self.__path_to_dataset_vad = os.path.normpath(path.strip())

    @property
    def path_to_dataset_vosk_sr(self) -> str:
        """Получение/установка пути к директории набора данных состоящего из фрагментов аудиовизуального сигнала (VOSK)

        Args:
            (str): Путь

        Returns:
            str: Путь
        """

        return self.__path_to_dataset_vosk_sr

    @path_to_dataset_vosk_sr.setter
    def path_to_dataset_vosk_sr(self, path: str):
        """Установка пути к директории набора данных состоящего из фрагментов аудиовизуального сигнала (VOSK)"""

        try:
            # Проверка аргументов
            if type(path) is not str or not path:
                raise TypeError
        except TypeError:
            pass
        else:
            self.__path_to_dataset_vosk_sr = os.path.normpath(path.strip())

    @property
    def ignore_dirs(self) -> List[str]:
        """Получение/установка списка с директориями не входящими в выборку

        Args:
            (List[str]): Список с директориями

        Returns:
            List[str]: Список с директориями
        """

        return self.__ignore_dirs

    @ignore_dirs.setter
    def ignore_dirs(self, l: List[str]) -> None:
        """Установка списка с директориями не входящими в выборку"""

        if type(l) is list:
            try:
                self.__ignore_dirs = [x.strip() for x in l]
            except Exception:
                pass

    @property
    def dir_va_names(self) -> List[str]:
        """Получение/установка списка с названиями директорий для видео и аудио

        Args:
            (List[str]): Список с директориями

        Returns:
            List[str]: Список с директориями
        """

        return self.__dir_va_names

    @dir_va_names.setter
    def dir_va_names(self, l: List[str]) -> None:
        """Установка списка с названиями директорий для видео и аудио"""

        if type(l) is list and len(l) == 2:
            try:
                self.__dir_va_names = [x.strip() for x in l]
            except Exception:
                pass

    @property
    def ext_search_files(self) -> List[str]:
        """Получение/установка списка с расширениями искомых файлов

        Args:
            (List[str]): Список с расширениями искомых файлов

        Returns:
            List[str]: Список с расширениями искомых файлов
        """

        return self.__ext_search_files

    @ext_search_files.setter
    def ext_search_files(self, l: List[str]) -> None:
        """Установка списка с расширениями искомых файлов"""

        if type(l) is list and len(l) > 0:
            try:
                self.__ext_search_files = ["." + x.strip().lower() for x in l]
            except Exception:
                pass

    @property
    def chunk_size(self) -> int:
        """Получение/установка размера загрузки файла из сети за 1 шаг

        Args:
            (int): Размер загрузки файла из сети за 1 шаг

        Returns:
            int: Размер загрузки файла из сети за 1 шаг
        """

        return self.__chunk_size

    @chunk_size.setter
    def chunk_size(self, size: int) -> None:
        """Установка директории для сохранения данных"""

        if type(size) is int and size > 0:
            self.__chunk_size = size



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Мультимодальное объединение аудио- и видеомодальностей
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
# Подавление Warning
import warnings

for warn in [
    UserWarning,
    FutureWarning,
]:
    warnings.filterwarnings(
        "ignore",
        category=warn,
    )

import os
import numpy as np

from dataclasses import dataclass  # Класс данных

# Типы данных
from typing import List, Dict

import datetime
import torch
from pathlib import Path  # Работа с путями в файловой системе
from torch.utils.data import DataLoader
from torch.nn import CrossEntropyLoss
from lion_pytorch import Lion
from tqdm import tqdm
from sklearn.metrics import accuracy_score

# Персональные
from openav.modules.core.exceptions import IsNestedCatalogsNotFoundError
from openav.modules.lab.audio import Audio  # Аудиомодальность
from openav.modules.lab.video import Video  # Видеомодальность
from openav.modules.nn.av_dataset import AVDataset, AVTest
from openav.modules.nn.utils import fix_seeds, train_one_epoch, val_one_epoch, save_conf_matrix
from openav.modules.nn.models import AVModel

# ######################################################################################################################
# Константы
# ######################################################################################################################

SUBFOLDERS: List[str] = ["train", "val", "test"]
SUBFOLDERS_TEST: List[str] = ["test"]
SHAPE_AUDIO: List[str] = ["channels", "n_mels", "samples"]
SHAPE_VIDEO: List[str] = ["frames", "channels", "width", "height"]
FIGSIZE_CONFUSION_MATRIX: List[str] = ["width", "height", "font_size", "dpi", "pad_inches"]
EXT_AV_VIDEO: List[str] = ["mov", "mp4", "webm"]  # Расширения искомых файлов
EXT_MODELS: str = "pt"
EXTH_MODELS: str = "pth"
OPTIMIZERS: List[str] = ["adam", "adamw", "sgd", "lion"]
REQUIRED_GRAD: List[str] = ["none", "a", "v", "av"]
CLASSES_TEST: List[str] = [
    "1_Позвонить",
    "2_Набрать_номер",
    "3_Отправить_сообщение",
    "6_Завершить_вызов",
    "7_Радио",
    "8_Музыка",
    "9_Воспроизвести",
    "11_Случа--и--ны--и--_выбор",
    "12_Отключить_случа--и--ны--и--_выбор",
    "13_Повтор",
    "16_Избегать_платных_дорог",
    "17_Карта",
    "19_Предыдущие_места_назначения",
    "20_Как_там_на_дорогах",
    "22_Надолго_пробка",
    "24_На_работу",
    "25_Максимальное_увеличение",
    "27_Остановить_маршрут",
    "28_Возобновить_маршрут",
    "29_Сколько_мне_еще_ехать",
    "30_Во_сколько_я_приеду",
    "32_Сбросить_маршрут",
    "34_На--и--ти_больницу",
    "36_На--и--ти_аптеку",
    "37_На--и--ти_банк",
    "39_На--и--ти_ресторан",
    "42_На--и--ти_железнодорожны--и--_вокзал",
    "47_Нет",
    "48_Предыдущая",
    "49_Следующая",
]


# ######################################################################################################################
# Сообщения
# ######################################################################################################################
@dataclass
class AVMessages(Audio, Video):
    """Класс для сообщений

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._subfolder_search: str = (
            self._('Поиск вложенной директории "{}" в директории "{}" (глубина вложенности: {})') + self._em
        )
        self._subfolders_search: str = (
            self._('Поиск вложенных директорий в директории "{}" (глубина вложенности: {})') + self._em
        )

        self._subfolder_not_found: str = self._("В указанной директории вложенная директория не найдена") + self._em

        self._files_audiovisual_find: str = (
            self._('Поиск файлов с расширениями "{}" в директории "{}" (глубина вложенности: {})') + self._em
        )

        self._files_audiovisual_find: str = (
            self._('Поиск файлов с расширениями "{}" в директории "{}" (глубина вложенности: {})') + self._em
        )

        self._sampling_nn: str = (
            self._("Разбиение найденных файлов на выборки (обучающая, валидационная, тестовая)") + self._em
        )

        self._sampling_nn_error: str = (
            self._(
                "Минимум одна выборка пустая (обучающая - {}, валидационная - {}, тестовая - {}) или количество меток "
                + "не совпадает с количеством файлов"
            )
            + self._em
        )

        self._sampling_nn_test_error: str = (
            self._("Тестовая выборка пустая или количество меток не совпадает с количеством файлов") + self._em
        )

        self._sampling_nn_true: str = self._("Обучающая - {} {}, валидационная - {} {}, тестовая - {} {}") + self._em
        self._format_percentage = lambda x: "({}%)".format(x)

        self._run_train: str = self._("Запуск процесса обучения") + self._em
        self._epoch: str = self._("Эпоха: {} из {}") + self._em
        self._loss: str = self._("Значения ошибки: обучение - {}, валидация - {}, тест - {}")
        self._acc_valid_and_test: str = self._("Валидация: точность {} | Тест: точность {}")
        self._acc_valid_up: str = (
            self._("Точность на валидационной выборке увеличилась ({} ---> {}). Сохранение модели") + self._em
        )
        self._acc_test: str = self._("Точность для тестовой выборке: {}")
        self._acc_test_up: str = self._("Точность на тестовой выборке увеличилась ({} ---> {})")

        self._end_train: str = self._("Процесс обучения завершен") + self._em

        self._run_test: str = self._("Запуск процесса тестирования") + self._em

        self._accuracy_score: str = self._("Точность на тестовой выборке: {}") + self._em
        self._create_confusion_matrix: str = self._("Создание матрицы спутывания") + self._em
        self._end_test: str = self._("Процесс тестирования завершен") + self._em


# ######################################################################################################################
# Мультимодальное
# ######################################################################################################################
@dataclass
class AV(AVMessages):
    """Класс для мультимодального объединения аудио- и видеомодальностей

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        # ----------------------- Только для внутреннего использования внутри класса

        self.__device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")

        self.__model_session = "created_at_%d_%m_%Y_%H-%M"
        self.__cm_session = "confusion_matrix_%d_%m_%Y_%H-%M.png"

    # ------------------------------------------------------------------------------------------------------------------
    # Свойства
    # ------------------------------------------------------------------------------------------------------------------

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (приватные)
    # ------------------------------------------------------------------------------------------------------------------

    def __get_hierarchy_from_paths(self, paths, num_levels=0):
        hierarchies = []

        for path in paths:
            hierarchy = []
            while True:
                path, dir_name = os.path.split(path)
                if not dir_name:
                    hierarchy.append(path)
                    break
                hierarchy.append(dir_name)
            if num_levels > 0:
                hierarchy = hierarchy[:num_levels]
            if hierarchy[-1] in SUBFOLDERS:
                hierarchies.append(hierarchy)

        return hierarchies

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (защищенные)
    # ------------------------------------------------------------------------------------------------------------------

    # ------------------------------------------------------------------------------------------------------------------
    # Внешние методы
    # ------------------------------------------------------------------------------------------------------------------

    def train_audiovisual(
        self,
        subfolders: Dict[str, str],
        n_classes: int,
        classes: List[str],
        encoder_decoder: int,
        batch_size: int,
        max_segment: int,
        patience: int,
        epochs: int,
        seed: int,
        leaning_rate: float,
        weight_decay: float,
        optimizer: str,
        hidden_units: int,
        hidden_features: int,
        input_dim: int,
        shape_audio: Dict[str, int],
        shape_video: Dict[str, int],
        path_to_model_fa: str,
        path_to_model_fv: str,
        requires_grad: str,
        path_to_save_models: str,
        out: bool = True,
    ) -> bool:
        """Автоматическое обучение на аудиовизуальных данных

        Args:
            subfolders (Dict[str, str]): Словарь с подкаталогами с данными
            n_classes (int): Количество классов
            classes (List[str]): Список классов
            encoder_decoder (int): Количество энкодеров и декодеров
            batch_size (int): Размер батча
            max_segment (int): Максимальная длительность сегмента видео
            patience (int): Количество неудачных эпох
            epochs (int): Количество эпох
            seed (int): Начальное состояние обучения
            leaning_rate (float): Скорость обучения
            weight_decay (float): Скорость обучения
            optimizer (str): Оптимизатор
            hidden_units (int): Количество скрытых нейронов
            hidden_features (int): Количество скрытых признаков
            input_dim (int): Количество входных признаков
            shape_audio (Dict[str, int]): Входная размерность аудио лог-мел спектрограммы
            shape_video (Dict[str, int]): Входная размерность видеокадров
            path_to_model_fa (str): Путь к нейросетевой модели (аудио)
            path_to_model_fv (str): Путь к нейросетевой модели (видео)
            path_to_save_models (str): Путь к директории для сохранения моделей
            requires_grad (str): Заморозка слоев для извлечения ауди и видео признаков
            out (bool) Отображение

        Returns:
            bool: **True** если автоматическое обучение на аудиовизуальных данных произведено, в обратном случае
            **False**
        """

        try:
            # Проверка аргументов
            if (
                type(subfolders) is not dict
                or len(subfolders) == 0
                or not all(subfolder in subfolders for subfolder in SUBFOLDERS)
                or type(n_classes) is not int
                or not (0 < n_classes)
                or type(classes) is not list
                or len(classes) == 0
                or type(encoder_decoder) is not int
                or not (1 <= encoder_decoder <= 50)
                or type(batch_size) is not int
                or not 0 <= batch_size
                or type(max_segment) is not int
                or not (1 <= max_segment <= 10)
                or type(epochs) is not int
                or not (0 <= epochs <= 1000)
                or type(seed) is not int
                or not (0 < seed)
                or type(leaning_rate) is not float
                or type(weight_decay) is not float
                or type(hidden_units) is not int
                or type(optimizer) is not str
                or (optimizer in OPTIMIZERS) is False
                or not (0 < hidden_units)
                or type(hidden_features) is not int
                or not (0 < hidden_features)
                or type(patience) is not int
                or not (0 < patience)
                or type(input_dim) is not int
                or not (0 < input_dim)
                or type(shape_audio) is not dict
                or len(shape_audio) == 0
                or not all(shape in shape_audio for shape in SHAPE_AUDIO)
                or type(shape_video) is not dict
                or len(shape_video) == 0
                or not all(shape in shape_video for shape in SHAPE_VIDEO)
                or type(path_to_model_fa) is not str
                or not path_to_model_fa
                or type(path_to_model_fv) is not str
                or not path_to_model_fv
                or not Path(path_to_model_fa).is_file()
                or Path(path_to_model_fa).suffix.replace(".", "") != EXT_MODELS
                or not Path(path_to_model_fv).is_file()
                or Path(path_to_model_fv).suffix.replace(".", "") != EXT_MODELS
                or type(requires_grad) is not str
                or (requires_grad in REQUIRED_GRAD) is False
                or type(path_to_save_models) is not str
                or not path_to_save_models
                or type(out) is not bool
            ):
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.train_audiovisual.__name__, out=out)
            return False
        else:
            depth = 3
            classes = [cls.lower() for cls in classes]
            optimizer = optimizer.lower()

            shape_audio = (
                None,
                max_segment,
                shape_audio[SHAPE_AUDIO[0]],
                shape_audio[SHAPE_AUDIO[1]],
                shape_audio[SHAPE_AUDIO[2]],
            )
            shape_video_frames = shape_video[SHAPE_VIDEO[0]]
            shape_video = (
                None,
                max_segment,
                shape_video[SHAPE_VIDEO[0]],
                shape_video[SHAPE_VIDEO[1]],
                shape_video[SHAPE_VIDEO[3]],
                shape_video[SHAPE_VIDEO[2]],
            )

            # Информационное сообщение
            self.message_info(
                self._subfolders_search.format(
                    self.message_line(self.path_to_dataset),
                    self.message_line(str(depth)),
                ),
                out=out,
            )

            nested_paths = self.get_paths(self.path_to_dataset, depth=depth, out=False)

            # Вложенные директории не найдены
            try:
                if len(nested_paths) == 0:
                    raise IsNestedCatalogsNotFoundError
            except IsNestedCatalogsNotFoundError:
                self.message_error(self._subfolders_not_found, space=self._space, out=out)
                return False

            # Информационное сообщение
            self.message_info(
                self._files_audiovisual_find.format(
                    self.message_line(", ".join(x.replace(".", "") for x in EXT_AV_VIDEO)),
                    self.message_line(self.path_to_dataset),
                    self.message_line(str(depth)),
                ),
                out=out,
            )

            hierarchy_from_paths = self.__get_hierarchy_from_paths(nested_paths, depth)

            # Информационное сообщение
            self.message_info(
                self._sampling_nn,
                out=out,
            )

            path_train, lb_train = [], []
            path_val, lb_val = [], []
            path_test, lb_test = [], []

            # Проход по всем вложенным директориям
            for nested_path in hierarchy_from_paths:
                nested_path_0 = nested_path[0].replace("--и--", "й").lower()
                if nested_path_0 in classes:
                    # Формирование списка с видеофайлами
                    for p in Path(os.path.join(self.path_to_dataset, *reversed(nested_path))).glob("*"):
                        # Добавление текущего пути к видеофайлу в список
                        if p.suffix.lower().replace(".", "") in EXT_AV_VIDEO:
                            [index for index, cls in enumerate(classes) if cls.lower() == nested_path_0]

                            if nested_path[-1] == subfolders[SUBFOLDERS[0]]:
                                path_train.append(p.resolve())
                                lb_train.append(classes.index(nested_path_0))
                            elif nested_path[-1] == subfolders[SUBFOLDERS[1]]:
                                if (nested_path[0] in CLASSES_TEST) is True:
                                    path_val.append(p.resolve())
                                    lb_val.append(classes.index(nested_path_0))
                            elif nested_path[-1] == subfolders[SUBFOLDERS[2]]:
                                if (nested_path[0] in CLASSES_TEST) is True:
                                    path_test.append(p.resolve())
                                    lb_test.append(classes.index(nested_path_0))
                            else:
                                pass

            # Директории с поднаборами данных не содержат визуальных файлов с необходимыми расширениями
            try:
                len_path_train = len(path_train)
                len_lb_train = len(lb_train)

                len_path_val = len(path_val)
                len_lb_val = len(lb_val)

                len_path_test = len(path_test)
                len_lb_test = len(lb_test)

                if (
                    len_path_train == 0
                    or len_lb_train == 0
                    or len_path_val == 0
                    or len_lb_val == 0
                    or len_path_test == 0
                    or len_lb_test == 0
                    or len_path_train != len_lb_train
                    or len_path_val != len_lb_val
                    or len_path_test != len_lb_test
                ):
                    raise ValueError
            except ValueError:
                self.message_error(
                    self._sampling_nn_error.format(
                        self.message_line(str(len_path_train)),
                        self.message_line(str(len_path_val)),
                        self.message_line(str(len_lb_test)),
                    ),
                    space=self._space,
                    out=out,
                )
                return False
            except Exception:
                self.message_error(self._unknown_err, space=self._space, out=out)
                return False
            else:
                total_samples = len_path_train + len_path_val + len_path_test

                train_percentage = round((len_path_train / total_samples) * 100, 2)
                val_percentage = round((len_path_val / total_samples) * 100, 2)
                test_percentage = round((len_path_test / total_samples) * 100, 2)

                self.message_info(
                    self._sampling_nn_true.format(
                        self.message_line(str(len_path_train)),
                        self.message_line(self._format_percentage(train_percentage)),
                        self.message_line(str(len_path_val)),
                        self.message_line(self._format_percentage(val_percentage)),
                        self.message_line(str(len_path_test)),
                        self.message_line(self._format_percentage(test_percentage)),
                    ),
                    space=self._space,
                    out=out,
                )

                train_data = AVDataset(
                    path_files=path_train,
                    labels=lb_train,
                    subset="train",
                    len_video=shape_video_frames,
                    max_segment=max_segment,
                )

                train_data = AVDataset(path_files=path_train, labels=lb_train, subset="train", max_segment=max_segment)
                test_data = AVDataset(path_files=path_test, labels=lb_test, subset="test", max_segment=max_segment)
                val_data = AVDataset(path_files=path_val, labels=lb_val, subset="val", max_segment=max_segment)

                train_dataloader = DataLoader(train_data, batch_size=batch_size, shuffle=True)
                val_dataloader = DataLoader(val_data, batch_size=batch_size, shuffle=False)
                test_dataloader = DataLoader(test_data, batch_size=batch_size, shuffle=False)

                torch.autograd.set_detect_anomaly(True)

                stop_flag_training = 0

                fix_seeds(seed)

                model = AVModel(
                    shape_audio=shape_audio,
                    shape_video=shape_video,
                    input_dim=input_dim,
                    h_u=hidden_units,
                    h_f=hidden_features,
                    n_class=n_classes,
                    encoder_decoder=encoder_decoder,
                ).to(self.__device)
                model.feature_audio.load_state_dict(torch.load(path_to_model_fa))
                model.feature_video.load_state_dict(torch.load(path_to_model_fv))

                if requires_grad == REQUIRED_GRAD[0]:
                    for name, param in model.named_parameters():
                        param.requires_grad = True
                elif requires_grad == REQUIRED_GRAD[1]:
                    for name, param in model.named_parameters():
                        if any(layer_name.split(".")[0] in name for layer_name in ["feature_audio"]):
                            param.requires_grad = False
                elif requires_grad == REQUIRED_GRAD[2]:
                    for name, param in model.named_parameters():
                        if any(layer_name.split(".")[0] in name for layer_name in ["feature_video"]):
                            param.requires_grad = False
                elif requires_grad == REQUIRED_GRAD[3]:
                    for name, param in model.named_parameters():
                        if any(layer_name.split(".")[0] in name for layer_name in ["feature_audio", "feature_video"]):
                            param.requires_grad = False
                else:
                    for name, param in model.named_parameters():
                        param.requires_grad = True

                criterion = CrossEntropyLoss()

                if optimizer == OPTIMIZERS[0]:
                    optimizer = torch.optim.Adam(model.parameters(), lr=leaning_rate, weight_decay=weight_decay)
                elif optimizer == OPTIMIZERS[1]:
                    optimizer = torch.optim.AdamW(model.parameters(), lr=leaning_rate, weight_decay=weight_decay)
                elif optimizer == OPTIMIZERS[2]:
                    optimizer = torch.optim.SGD(model.parameters(), lr=leaning_rate, weight_decay=weight_decay)
                elif optimizer == OPTIMIZERS[3]:
                    optimizer = Lion(model.parameters(), lr=leaning_rate, weight_decay=weight_decay)
                else:
                    pass

                max_acc = 0
                max_test_acc = 0

                date_time = datetime.datetime.now()
                date_path = date_time.strftime(self.__model_session)
                session_path = os.path.join(self.path_to_save_models, date_path)

                # Создание директории, где хранятся модели
                if self.create_folder(session_path, out=False) is False:
                    return False

                self.message_info(self._run_train, out=out)

                for e in range(1, epochs + 1, 1):
                    self.message_info(
                        self._epoch.format(
                            self.message_line(str(e)),
                            self.message_line(str(epochs)),
                        ),
                        out=out,
                    )

                    model.train()
                    avg_loss = train_one_epoch(train_dataloader, optimizer, criterion, model, self.__device)

                    model.eval()
                    acc, avg_vloss = val_one_epoch(val_dataloader, criterion, model, self.__device)

                    model.eval()
                    test_acc, avg_tloss = val_one_epoch(test_dataloader, criterion, model, self.__device)

                    round_f = 6

                    self.message_info(
                        self._loss.format(
                            self.message_line(str(round(avg_loss, round_f))),
                            self.message_line(str(round(avg_vloss, round_f))),
                            self.message_line(str(round(avg_tloss, round_f))),
                        ),
                        out=out,
                    )

                    if max_acc < acc:
                        stop_flag_training = 0

                        self.message_info(
                            self._acc_valid_and_test.format(
                                self.message_line(str(round(acc, round_f))),
                                self.message_line(str(round(test_acc, round_f))),
                            ),
                            out=out,
                        )
                        self.message_info(
                            self._acc_valid_up.format(
                                self.message_line(str(round(max_acc, round_f))),
                                self.message_line(str(round(acc, round_f))),
                            ),
                            out=out,
                        )
                        max_acc = acc
                        torch.save(model.state_dict(), "{}/e{}_{:.6f}.pth".format(session_path, e, acc))

                    if max_test_acc < test_acc:
                        self.message_info(
                            self._acc_test.format(
                                self.message_line(str(round(test_acc, round_f))),
                            ),
                            out=out,
                        )
                        self.message_info(
                            self._acc_test_up.format(
                                self.message_line(str(round(max_test_acc, round_f))),
                                self.message_line(str(round(test_acc, round_f))),
                            ),
                            out=out,
                        )

                        max_test_acc = test_acc

                    else:
                        stop_flag_training += 1

                    if stop_flag_training > patience:
                        self.message_info(self._end_train, out=out)

                        return True

    def test_audiovisual(
        self,
        subfolders: Dict[str, str],
        n_classes: int,
        classes: List[str],
        encoder_decoder: int,
        max_segment: int,
        hidden_units: int,
        hidden_features: int,
        input_dim: int,
        shape_audio: Dict[str, int],
        shape_video: Dict[str, int],
        save_confusion_matrix: bool,
        path_to_save_confusion_matrix: str,
        figsize_confusion_matrix: Dict[str, int],
        path_to_model: str,
        out: bool = True,
    ) -> bool:
        """Автоматическое тестирование на аудиовизуальных данных

        Args:
            subfolders (Dict[str, str]): Словарь с подкаталогами с данными
            n_classes (int): Количество классов
            classes (List[str]): Список классов
            encoder_decoder (int): Количество энкодеров и декодеров
            max_segment (int): Максимальная длительность сегмента видео
            hidden_units (int): Количество скрытых нейронов
            hidden_features (int): Количество скрытых признаков
            input_dim (int): Количество входных признаков
            shape_audio (Dict[str, int]): Входная размерность аудио лог-мел спектрограммы
            shape_video (Dict[str, int]): Входная размерность видеокадров
            save_confusion_matrix (bool): Сохранение матрицы спутывания
            path_to_save_confusion_matrix (str):  Путь к директории для сохранения матрицы спутывания
            figsize_confusion_matrix (Dict[str, int]): Настройки для формирования изображения матрицы спутывания
            path_to_model (str): Путь к нейросетевой аудиовизуальной модели
            out (bool) Отображение

        Returns:
            bool: **True** если автоматическое тестирование на аудиовизуальных данных произведено, в обратном случае
            **False**
        """

        try:
            # Проверка аргументов
            if (
                type(subfolders) is not dict
                or len(subfolders) == 0
                or not all(subfolder in subfolders for subfolder in SUBFOLDERS_TEST)
                or type(n_classes) is not int
                or not (0 < n_classes)
                or type(classes) is not list
                or len(classes) == 0
                or type(encoder_decoder) is not int
                or not (1 <= encoder_decoder <= 50)
                or type(max_segment) is not int
                or not (1 <= max_segment <= 10)
                or not (0 < hidden_units)
                or type(hidden_features) is not int
                or not (0 < hidden_features)
                or type(input_dim) is not int
                or not (0 < input_dim)
                or type(shape_audio) is not dict
                or len(shape_audio) == 0
                or not all(shape in shape_audio for shape in SHAPE_AUDIO)
                or type(shape_video) is not dict
                or len(shape_video) == 0
                or not all(shape in shape_video for shape in SHAPE_VIDEO)
                or type(save_confusion_matrix) is not bool
                or type(path_to_save_confusion_matrix) is not str
                or not path_to_save_confusion_matrix
                or len(figsize_confusion_matrix) == 0
                or not all(k in figsize_confusion_matrix for k in FIGSIZE_CONFUSION_MATRIX)
                or type(path_to_model) is not str
                or not path_to_model
                or type(out) is not bool
            ):
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.test_audiovisual.__name__, out=out)
            return False
        else:
            depth = 3
            classes = [cls.lower() for cls in classes]

            shape_audio = (
                None,
                max_segment,
                shape_audio[SHAPE_AUDIO[0]],
                shape_audio[SHAPE_AUDIO[1]],
                shape_audio[SHAPE_AUDIO[2]],
            )
            shape_video = (
                None,
                max_segment,
                shape_video[SHAPE_VIDEO[0]],
                shape_video[SHAPE_VIDEO[1]],
                shape_video[SHAPE_VIDEO[3]],
                shape_video[SHAPE_VIDEO[2]],
            )

            # Информационное сообщение
            self.message_info(
                self._subfolder_search.format(
                    self.message_line(subfolders[SUBFOLDERS_TEST[0]]),
                    self.message_line(self.path_to_dataset),
                    self.message_line(str(depth)),
                ),
                out=out,
            )

            nested_paths = self.get_paths(self.path_to_dataset, depth=depth, out=False)

            # Вложенные директории не найдены
            try:
                if len(nested_paths) == 0:
                    raise IsNestedCatalogsNotFoundError
            except IsNestedCatalogsNotFoundError:
                self.message_error(self._subfolder_not_found, space=self._space, out=out)
                return False

            # Информационное сообщение
            self.message_info(
                self._files_audiovisual_find.format(
                    self.message_line(", ".join(x.replace(".", "") for x in EXT_AV_VIDEO)),
                    self.message_line(self.path_to_dataset),
                    self.message_line(str(depth)),
                ),
                out=out,
            )

            hierarchy_from_paths = self.__get_hierarchy_from_paths(nested_paths, depth)

            path_test, lb_test = [], []

            # Проход по всем вложенным директориям
            for nested_path in hierarchy_from_paths:
                nested_path_0 = nested_path[0].replace("--и--", "й").lower()
                if nested_path_0 in classes:
                    # Формирование списка с видеофайлами
                    for p in Path(os.path.join(self.path_to_dataset, *reversed(nested_path))).glob("*"):
                        # Добавление текущего пути к видеофайлу в список
                        if p.suffix.lower().replace(".", "") in EXT_AV_VIDEO:
                            [index for index, cls in enumerate(classes) if cls.lower() == nested_path[0].lower()]

                            if nested_path[-1] == subfolders[SUBFOLDERS_TEST[0]]:
                                path_test.append(p.resolve())
                                lb_test.append(classes.index(nested_path_0))
                            else:
                                pass

            lb_test_names = [classes[val] for val in set(lb_test)]

            # Директории с поднаборами данных не содержат визуальных файлов с необходимыми расширениями
            try:
                len_path_test = len(path_test)
                len_lb_test = len(lb_test)

                if len_path_test == 0 or len_lb_test == 0 or len_path_test != len_lb_test:
                    raise ValueError
            except ValueError:
                self.message_error(
                    self._sampling_nn_test_error,
                    space=self._space,
                    out=out,
                )
                return False
            except Exception:
                self.message_error(self._unknown_err, space=self._space, out=out)
                return False
            else:
                model = AVModel(
                    shape_audio=shape_audio,
                    shape_video=shape_video,
                    input_dim=input_dim,
                    h_u=hidden_units,
                    h_f=hidden_features,
                    n_class=n_classes,
                    encoder_decoder=encoder_decoder,
                ).to(self.__device)

                model.load_state_dict(torch.load(path_to_model, map_location=torch.device(self.__device)))

                self.message_info(self._run_test, out=out)

                processor = AVTest(max_segment=max_segment)

                torch.autograd.set_detect_anomaly(True)

                model.eval().to(self.__device)

                preds = []

                for path in tqdm(path_test):
                    _, audio_data, video_data = processor.get_metadata(name_file=str(path))
                    with torch.no_grad():
                        prob = model(audio_data.to(self.__device), video_data.to(self.__device)).cpu().detach().numpy()
                    pred = np.argmax(prob)
                    preds.append(pred)

                self.message_info(
                    self._accuracy_score.format(
                        self.message_line(str(round(accuracy_score(lb_test, preds) * 100, 2)) + "%"),
                    ),
                    out=out,
                )

                if save_confusion_matrix:
                    self.message_info(self._create_confusion_matrix, out=out)

                    date_time = datetime.datetime.now()
                    date_path = date_time.strftime(self.__cm_session)

                    save_conf_matrix(
                        y_true=lb_test,
                        y_pred=preds,
                        name_labels=["_".join(val.split("_")[1:]).capitalize() for val in lb_test_names],
                        filename=os.path.join(path_to_save_confusion_matrix, date_path),
                        figsize_w=figsize_confusion_matrix[FIGSIZE_CONFUSION_MATRIX[0]],
                        figsize_h=figsize_confusion_matrix[FIGSIZE_CONFUSION_MATRIX[1]],
                        font_size=figsize_confusion_matrix[FIGSIZE_CONFUSION_MATRIX[2]],
                        dpi=figsize_confusion_matrix[FIGSIZE_CONFUSION_MATRIX[3]],
                        pad_inches=figsize_confusion_matrix[FIGSIZE_CONFUSION_MATRIX[4]],
                    )

                self.message_info(self._end_test, out=out)

                return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Работа с JSON
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
import os  # Работа с файловой системой
import json  # Кодирование и декодирование данные в удобном формате

import importlib.resources as pkg_resources  # Работа с ресурсами внутри пакетов

from dataclasses import dataclass  # Класс данных

from typing import Dict, Union  # Типы данных
from types import ModuleType

# Персональные
from openav.modules.file_manager.download import Download  # Загрузка файлов


# ######################################################################################################################
# Сообщения
# ######################################################################################################################
@dataclass
class JsonMessages(Download):
    """Класс для сообщений

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._load_data: str = self._('Загрузка данных из файла "{}"') + self._em
        self._invalid_file: str = self._("Данные не загружены") + self._em
        self._config_empty: str = self._("Файл пуст") + self._em
        self._load_data_resources: str = self._('Загрузка данных из ресурсов "{}"') + self._em
        self._load_data_resources_not_found: str = self._("Ресурс не найден") + self._em


# ######################################################################################################################
# Работа с JSON
# ######################################################################################################################
@dataclass
class Json(JsonMessages):
    """Класс для работы с JSON

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

    # ------------------------------------------------------------------------------------------------------------------
    #  Внешние методы
    # ------------------------------------------------------------------------------------------------------------------

    def load_json(
        self, path_to_file: str, create: bool = False, out: bool = True
    ) -> Dict[str, Union[str, bool, int, float,],]:
        """Загрузка JSON файла

        Args:
            path_to_file (str): Путь к файлу JSON
            create (bool): Создание файла JSON в случае его отсутствия
            out (bool): Печатать процесс выполнения

        Returns:
            Dict[str, Union[str, bool, int, float]]: Словарь из json файла
        """

        # Проверка аргументов
        if type(path_to_file) is not str or not path_to_file or type(create) is not bool or type(out) is not bool:
            self.inv_args(__class__.__name__, self.load_json.__name__, out=out)
            return {}

        # Поиск JSON файла не удался
        if self.search_file(path_to_file, "json", create, out) is False:
            return {}

        path_to_file = os.path.normpath(path_to_file)

        # Вывод сообщения
        self.message_info(
            self._load_data.format(self.message_line(os.path.basename(path_to_file))), space=self._space, out=out
        )

        # Открытие файла
        with open(path_to_file, mode="r", encoding="utf-8") as json_data_file:
            try:
                config = json.load(json_data_file)
            except json.JSONDecodeError:
                self.message_error(self._invalid_file, space=self._space, out=out)
                return {}

        # Файл пуст
        if not config:
            self.message_error(self._config_empty, space=self._space, out=out)
            return {}

        return config  # Результат

    def load_json_resources(
        self, module: ModuleType, path_to_file: str, out: bool = True
    ) -> Dict[str, Union[str, bool, int, float]]:
        """Загрузка JSON файла из ресурсов модуля

        Args:
            module (ModuleType): Модуль
            path_to_file (str): Путь к файлу JSON
            out (bool): Печатать процесс выполнения

        Returns:
            Dict[str, Union[str, bool, int, float]]: Словарь из json файла
        """

        # Проверка аргументов
        if (
            isinstance(module, ModuleType) is False
            or type(path_to_file) is not str
            or not path_to_file
            or type(out) is not bool
        ):
            self.inv_args(__class__.__name__, self.load_json_resources.__name__, out=out)
            return {}

        # Вывод сообщения
        self.message_info(self._load_data_resources.format(self.message_line(module.__name__)), out=out)

        # Ресурс с JSON файлом не найден
        if pkg_resources.is_resource(module, path_to_file) is False:
            self.message_error(self._load_data_resources_not_found, space=self._space, out=out)
            return {}

        # Открытие файла
        with pkg_resources.open_text(module, path_to_file, encoding="utf-8", errors="strict") as json_data_file:
            try:
                config = json.load(json_data_file)
            except json.JSONDecodeError:
                self.message_error(self._invalid_file, space=self._space, out=out)
                return {}

        # Файл пуст
        if not config:
            self.message_error(self._config_empty, space=self._space, out=out)
            return {}

        return config  # Результат



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Запись речевых аудиовизуальных данных
"""

import os
import sys

PATH_TO_SOURCE = os.path.abspath(os.path.dirname(__file__))
PATH_TO_ROOT = os.path.join(PATH_TO_SOURCE, "..", "..")

sys.path.insert(0, os.path.abspath(PATH_TO_ROOT))

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
# Подавление Warning
import warnings

for warn in [UserWarning, FutureWarning]:
    warnings.filterwarnings("ignore", category=warn)

from dataclasses import dataclass  # Класс данных

import logging  # Логирование Функции создающие итераторы для эффективного цикла

from flask import Flask, request, render_template, jsonify, send_file
from werkzeug.middleware.proxy_fix import ProxyFix

# Типы данных
from typing import Dict, Union, Any
from types import ModuleType

# Персональные
import openav  # Библиотека в целом
from openav.modules.trml.shell import Shell  # Работа с Shell
from openav.modules.lab.build import Run  # Сборка библиотеки
from openav import rsrs  # Ресурсы библиотеки

from openav.modules.core.logging import ARG_PATH_TO_LOGS


app = Flask(
    __name__,
    template_folder="../modules/dataset_recording/templates",
    static_folder="../modules/dataset_recording/static",
)
app.wsgi_app = ProxyFix(app.wsgi_app, x_for=1, x_proto=1, x_host=1, x_prefix=1)

global video_path, output_dir
global processing_finished

processing_finished = False
video_path = "temp_dir/video.mp4"
output_dir = "cuted"

global questions, path_to_temp_dir

path_to_temp_dir = ""

questions = []


@app.route("/get_questions", methods=["GET"])
def get_questions():
    """Получение списка вопросов

    GET /get_questions

    Возвращает список вопросов в json формате
    """

    return jsonify(questions=questions)


# Configure the upload folder for recorded videos
app.config["UPLOAD_FOLDER"] = "../modules/dataset_recording/static/recorded-video"

if not os.path.exists(app.config["UPLOAD_FOLDER"]):
    os.makedirs(app.config["UPLOAD_FOLDER"])


@app.route("/")
def index():
    """Отображение основной страницы записи

    GET /

    Отображает статику страницы записи. Является точкой входа в приложение
    """
    global processing_finished
    processing_finished = False
    return render_template("index_home.html")


@app.route("/store_timing_data", methods=["POST"])
def store_timing_data():
    """Сохранение файла временных отметок записи

    POST /store_timing_data

    Сохраняет файл временных отметок записи, который предается в виде json в теле запроса
    """
    if request.method == "POST":
        data = request.json
        if os.path.exists("timing_data.txt"):
            os.remove("timing_data.txt")
        with open("timing_data.txt", "w", encoding="utf-8") as file:
            for item in data:
                file.write(f"Question: {item['question']}, Timestamp: {item['timestamp']}\n")

            file.close()
        return "Data stored successfully"


@app.route("/upload", methods=["POST"])
def upload():
    """Сохранение файла записи

    POST /upload

    Сохраняет файл записи, который предается в виде файла видео в теле запроса
    """
    global processing_finished
    if "video" in request.files:
        video_file = request.files["video"]
        if video_file.filename != "":
            if not os.path.exists(path_to_temp_dir):
                os.makedirs(path_to_temp_dir)
            video_file.save(os.path.join(path_to_temp_dir, "video.webm"))

            processing_finished = True

            return "DONE"

    return "No video data received."


@app.route("/download_processed_video")
def download_processed_video():
    """Выгрузка файла записи

    GET /download_processed_video

    Скачивает файл с записью в формате webm
    """
    global path_to_temp_dir
    processed_video_path = os.path.join(path_to_temp_dir, "video.webm")

    return send_file(processed_video_path, as_attachment=True)


@app.route("/download_timing_data")
def download_timing_data():
    """Выгрузка файла временных отметок записи

    GET /download_processed_video

    Скачивает файл временных отметок записи в формате txt
    """
    timing_data_path = "timing_data.txt"  # Path to the file
    # Set cache-control headers to prevent caching
    response = send_file(timing_data_path, as_attachment=True, mimetype="text/plain")
    response.headers["Cache-Control"] = "no-store, no-cache, must-revalidate, max-age=0"
    return response


# ######################################################################################################################
# Сообщения
# ######################################################################################################################
@dataclass
class MessagesRecorder(Run):
    """Класс для сообщений"""

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._description: str = self._("Запись речевых аудиовизуальных данных")
        self._description_time: str = "{}" * 2 + self._description + self._em + "{}"

        self._check_config_file_valid = self._("Проверка данных на валидность") + self._em

        self._recorder_url = self._("Для записи перейдите по адресу: http://127.0.0.1:5000") + self._em


# ######################################################################################################################
# Выполняем только в том случае, если файл запущен сам по себе
# ######################################################################################################################
@dataclass
class RunRecorder(MessagesRecorder):
    """Класс для записи речевых аудиовизуальных данных"""

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._all_layer_in_yaml = 5  # Общее количество настроек в конфигурационном файле

        #  Регистратор логирования с указанным именем
        self._logger_run_train: logging.Logger = logging.getLogger(__class__.__name__)

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (защищенные)
    # ------------------------------------------------------------------------------------------------------------------

    def _build_args(self, description: str, conv_to_dict: bool = True, out=True) -> Dict[str, Any]:
        """
        Args:
            description (str): Описание парсера командной строки
            conv_to_dict (bool): Преобразование списка аргументов командной строки в словарь
            out (bool): Печатать процесс выполнения

        Returns:
            Dict[str, Any]: Словарь со списком аргументов командной стройки
        """

        # Выполнение функции из суперкласса
        super().build_args(description=description, conv_to_dict=False, out=out)

        if self._ap is None:
            return {}

        # Добавление аргументов в парсер командной строки
        self._ap.add_argument(
            "--config", required=True, metavar=self._("ФАЙЛ"), help=self._("Путь к конфигурационному файлу")
        )

        self._ap.add_argument(
            ARG_PATH_TO_LOGS,
            required=False,
            metavar=self._("ФАЙЛ"),
            help=self._("Путь к директории для сохранения LOG файлов"),
        )

        self._ap.add_argument(
            "--automatic_update",
            action="store_true",
            help=self._(
                "Автоматическая проверка конфигурационного файла в момент работы программы (работает при заданном"
            )
            + " --config",
        )
        self._ap.add_argument(
            "--no_clear_shell", action="store_false", help=self._("Не очищать консоль перед выполнением")
        )

        # Преобразование списка аргументов командной строки в словарь
        if conv_to_dict is True:
            args, _ = self._ap.parse_known_args()
            return vars(args)  # Преобразование списка аргументов командной строки в словарь

    def _valid_yaml_config(self, config: Dict[str, Union[str, bool, int, float]], out: bool = True) -> bool:
        """Проверка настроек JSON на валидность

        Args:
            config (Dict[str, Union[str, bool, int, float]]): Словарь из JSON файла
            out (bool): Печатать процесс выполнения

        Returns:
             bool: **True** если конфигурационный файл валиден, в обратном случае **False**
        """

        # Проверка аргументов
        if type(config) is not dict or type(out) is not bool:
            try:
                raise TypeError
            except TypeError:
                self.inv_args(__class__.__name__, self._valid_yaml_config.__name__, out=out)
                return False

        # Конфигурационный файл пуст
        if not config:
            try:
                raise TypeError
            except TypeError:
                self.message_error(self._config_empty, space=self._space, out=out)
                return False

        # Вывод сообщения
        self.message_info(self._check_config_file_valid, space=self._space, out=out)

        curr_valid_layer = 0  # Валидное количество разделов

        # Проход по всем разделам конфигурационного файла
        for key, val in config.items():
            # 1. Скрытие метаданных
            # 2. Скрытие версий установленных библиотек
            if key == "hide_metadata" or key == "hide_libs_vers":
                # Проверка значения
                if type(val) is not bool:
                    continue

                curr_valid_layer += 1

            # Словарь
            if key == "dictionary":
                # Проверка значения
                if type(val) is not list or len(val) == 0:
                    continue

                # Проход по всем классам
                for v in val:
                    # Проверка значения
                    if type(v) is not str or not v:
                        continue

            # Задержка
            if key == "sleep":
                # Проверка значения
                if type(val) is not int or not (0 <= val <= 10000):
                    continue

                curr_valid_layer += 1

                curr_valid_layer += 1

            # 1. Путь к временной директории
            if key == "path_to_temp_dir":
                # Проверка значения
                if type(val) is not str or not val:
                    continue

                curr_valid_layer += 1

        # Сравнение общего количества ожидаемых настроек и валидных настроек в конфигурационном файле
        if self._all_layer_in_yaml != curr_valid_layer:
            try:
                raise TypeError
            except TypeError:
                self.message_error(self._invalid_file, space=self._space, out=out)
                return False

        return True  # Результат

    def _load_config_yaml(self, resources: ModuleType = rsrs, config="recorder.yaml", out: bool = True) -> bool:
        """Загрузка и проверка конфигурационного файла

        Args:
            resources (ModuleType): Модуль с ресурсами
            config (str): Конфигурационный файл
            out (bool): Печатать процесс выполнения

        Returns:
             bool: **True** если конфигурационный файл загружен и валиден, в обратном случае **False**
        """

        # Проверка аргументов
        if not isinstance(resources, ModuleType) or type(config) is not str or not config or type(out) is not bool:
            try:
                raise TypeError
            except TypeError:
                self.inv_args(__class__.__name__, self._load_config_yaml.__name__, out=out)
                return False

        # Конфигурационный файл передан
        if self._args["config"] is not None:
            config_yaml = self.load_yaml(self._args["config"], False, out)  # Загрузка YAML файла
        else:
            config_yaml = self.load_yaml_resources(resources, config, out)  # Загрузка YAML файла из ресурсов модуля

        # Конфигурационный файл не загружен
        if not config_yaml:
            return False

        # Проверка конфигурационного файла на валидность
        res_valid_yaml_config = self._valid_yaml_config(config_yaml, out)

        # Конфигурационный файл не валидный
        if res_valid_yaml_config is False:
            return False

        # Проход по всем разделам конфигурационного файла
        for k, v in config_yaml.items():
            self._args[k] = v  # Добавление значения из конфигурационного файла в словарь аргументов командной строки

        return True

    # ------------------------------------------------------------------------------------------------------------------
    #  Внешние методы
    # ------------------------------------------------------------------------------------------------------------------

    def run(self, metadata: ModuleType = openav, resources: ModuleType = rsrs, out: bool = True) -> bool:
        """Запуск записи речевых аудиовизуальных данных

        Args:
            metadata (ModuleType): Модуль из которого необходимо извлечь информацию
            resources (ModuleType): Модуль с ресурсами
            out (bool): Печатать процесс выполнения

        Returns:
             bool: **True** если процесс записи речевых аудиовизуальных данных произведен успешно,
                   в обратном случае **False**
        """

        # Проверка аргументов
        if not isinstance(metadata, ModuleType) or not isinstance(resources, ModuleType) or type(out) is not bool:
            try:
                raise TypeError
            except TypeError:
                self.inv_args(__class__.__name__, self.run.__name__, out=out)
                return False

        self._args = self._build_args(self._description)  # Построение аргументов командной строки
        if len(self._args) == 0:
            return False

        # Очистка консоли перед выполнением
        if self.clear_shell(cls=self._args["no_clear_shell"], out=True) is False:
            return False

        # Вывод сообщения
        if out is True:
            # Приветствие
            Shell.add_line()  # Добавление линии во весь экран
            print(self._description_time.format(self.text_bold, self.color_blue, self.text_end))
            self._logger_run_train.info(self._description)
            Shell.add_line()  # Добавление линии во весь экран

        # Загрузка и проверка конфигурационного файла
        if self._load_config_yaml(resources, out=out) is False:
            return False

        # Вывод сообщения
        if out is True:
            Shell.add_line()  # Добавление линии во весь экран

        # Информация об библиотеке
        if self._args["hide_metadata"] is False and out is True:
            self.message_metadata_info(out=out)
            Shell.add_line()  # Добавление линии во весь экран

        # Версии установленных библиотек
        if self._args["hide_libs_vers"] is False and out is True:
            self.libs_vers(out=out)
            Shell.add_line()  # Добавление линии во весь экран

        self.message_info(
            self._recorder_url,
            out=out,
        )

        for cnt, d in enumerate(self._args["dictionary"], start=1):
            questions.append({"QuestionNumber": cnt, "QuestionText": d, "Disable_time": self._args["sleep"]})

        path_to_temp_dir = self._args["path_to_temp_dir"]

        app.run(debug=False)

        return True


def main():
    # Запуск процесса записи
    recorder = RunRecorder(lang="ru", path_to_logs="./openav/logs")
    recorder.run(out=True)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Загрузка файлов
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
# Подавление Warning
import warnings

for warn in [UserWarning, FutureWarning]:
    warnings.filterwarnings("ignore", category=warn)

from dataclasses import dataclass  # Класс данных

import os  # Взаимодействие с файловой системой
import numpy as np  # Научные вычисления
import requests  # Отправка HTTP запросов
import re  # Регулярные выражения
import shutil  # Набор функций высокого уровня для обработки файлов, групп файлов, и папок

from pathlib import Path  # Работа с путями в файловой системе

# Персональные
from openav.modules.core.exceptions import InvalidContentLength
from openav.modules.file_manager.file_manager import FileManager  # Работа с файлами

from openav.modules.core.core import TYPE_MESSAGES  # Типы возможных сообщений


# ######################################################################################################################
# Сообщения
# ######################################################################################################################
@dataclass
class DownloadMessages(FileManager):
    """Класс для сообщений

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._could_not_process_url = self._("Не удалось обработать указанный URL") + self._em
        self._url_incorrect = self._("URL указан некорректно") + self._em
        self._url_incorrect_content_length = self._("Не определен размер файла для загрузки") + self._em
        self._automatic_download: str = self._("Загрузка файла") + ' "{}"' + self._em
        self._automatic_download_progress: str = self._automatic_download + self._download_precent + self._em
        self._url_error_code_http: str = "(" + self._("ошибка") + " {})"
        self._url_error_http: str = self._("Не удалось скачать файл") + ' "{}" {}' + self._em


# ######################################################################################################################
# Загрузка файлов
# ######################################################################################################################
class Download(DownloadMessages):
    """Класс для загрузки файлов

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._headers: str = (
            "Mozilla/5.0 (Macintosh; Intel Mac OS X 11_2_3) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/89.0.4389.90 Safari/537.36"
        )  # User-Agent

        self._url_last_filename: str = ""  # Имя последнего загруженного файла

    # ------------------------------------------------------------------------------------------------------------------
    # Внешние методы
    # ------------------------------------------------------------------------------------------------------------------

    def download_file_from_url(self, url: str, force_reload: bool = True, out: bool = True) -> int:
        """Загрузка файла из URL

        Args:
            url (str): Полный путь к файлу
            force_reload (bool): Принудительная загрузка файла из сети
            out (bool): Отображение

        Returns:
            int: Код статуса ответа:

                * ``200`` - Файл загружен
                * ``400`` - Ошибка при проверке аргументов
                * ``404`` - Не удалось скачать файл
        """

        try:
            # Проверка аргументов
            if type(url) is not str or not url or type(force_reload) is not bool or type(out) is not bool:
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.download_file_from_url.__name__, out=out)
            return 400
        else:
            try:
                # Отправка GET запроса для получения файла
                r = requests.get(url, headers={"user-agent": self._headers}, stream=True)
            except (
                # https://requests.readthedocs.io/en/master/_modules/requests/exceptions/
                requests.exceptions.MissingSchema,
                requests.exceptions.InvalidSchema,
                requests.exceptions.ConnectionError,
                requests.exceptions.InvalidURL,
            ):
                self.message_error(self._could_not_process_url, out=out)
                return 404
            except Exception:
                self.message_error(self._unknown_err, out=out)
                return 404
            else:
                # Имя файла
                if "Content-Disposition" in r.headers.keys():
                    try:
                        url_filename = re.findall(r'(?<=[\(\{\["]).+(?=[\)\}\]"])', r.headers["Content-Disposition"])[0]
                    except IndexError:
                        url_filename = re.findall(
                            r'filename\*?=[\'"]?(?:UTF-\d[\'"]*)?([^;\r\n"\']*)[\'"]?;?',
                            r.headers["Content-Disposition"],
                        )[0]
                else:
                    url_filename = url.split("/")[-1]

                try:
                    # URL файл невалидный
                    if not url_filename or not Path(url_filename).suffix:
                        if not Path(url_filename).stem.lower():
                            raise requests.exceptions.InvalidURL

                        if r.headers["Content-Type"] == "image/jpeg":
                            ext = "jpg"
                        elif r.headers["Content-Type"] == "image/png":
                            ext = "png"
                        elif r.headers["Content-Type"] == "text/plain":
                            ext = "txt"
                        elif r.headers["Content-Type"] == "text/csv":
                            ext = "csv"
                        elif r.headers["Content-Type"] == "video/mp4":
                            ext = "mp4"
                        else:
                            raise requests.exceptions.InvalidHeader

                        url_filename = Path(url_filename).stem + "." + ext
                except (requests.exceptions.InvalidURL, requests.exceptions.InvalidHeader):
                    self.message_error(self._url_incorrect, out=out)
                    return 404
                except Exception:
                    self.message_error(self._unknown_err, out=out)
                    return 404
                else:
                    # Создание директории для сохранения файла
                    if self.create_folder(self.path_to_save_models, out=False) is False:
                        return 404

                    # Информационное сообщение
                    self.message_info(
                        self._automatic_download.format(self.message_line(url_filename)), end=False, out=out
                    )

                    local_file = os.path.join(self.path_to_save_models, url_filename)  # Путь к файлу

                    try:
                        # Принудительная загрузка файла из сети
                        if force_reload is True:
                            # Файл найден
                            if os.path.isfile(local_file) is True:
                                # Удаление файла
                                try:
                                    shutil.rmtree(local_file)
                                except OSError:
                                    os.remove(local_file)
                                except Exception:
                                    raise Exception
                    except Exception:
                        self.message_error(self._unknown_err, space=self._space, start=True, out=out)
                        return 404
                    else:
                        # Файл с указанным именем найден локально и принудительная загрузка файла из сети не указана
                        if Path(local_file).is_file() is True and force_reload is False:
                            self._url_last_filename = local_file
                            if out is True:
                                print()
                            return 200
                        else:
                            # Ответ получен
                            if r.status_code == 200:
                                total_length = int(r.headers.get("content-length", 0))  # Длина файла

                                try:
                                    if total_length == 0:
                                        raise InvalidContentLength
                                except InvalidContentLength:
                                    self.message_error(
                                        self._url_incorrect_content_length, space=self._space, start=True, out=out
                                    )
                                    return 404
                                else:
                                    num_bars = int(np.ceil(total_length / self.chunk_size))  # Количество загрузок

                                    try:
                                        # Открытие файла для записи
                                        with open(local_file, "wb") as f:
                                            # Сохранение файла по частям
                                            for i, chunk in enumerate(r.iter_content(chunk_size=self.chunk_size)):
                                                f.write(chunk)  # Запись в файл
                                                f.flush()

                                                self.message_progressbar(
                                                    self._automatic_download_progress.format(
                                                        self.message_line(url_filename), round(i * 100 / num_bars, 2)
                                                    ),
                                                    out=out,
                                                )

                                            self.message_progressbar(
                                                self._automatic_download_progress.format(
                                                    self.message_line(url_filename), 100
                                                ),
                                                out=out,
                                            )
                                    except Exception:
                                        self.message_error(self._unknown_err, space=self._space, start=True, out=out)
                                        return 404
                                    else:
                                        self._url_last_filename = local_file
                                        if out is True:
                                            print()
                                        return 200
                            else:
                                self.message_error(
                                    self._url_error_http.format(
                                        self.message_line(url_filename, TYPE_MESSAGES[2]),
                                        self._url_error_code_http.format(
                                            self.message_line(str(r.status_code), type_message=TYPE_MESSAGES[2])
                                        ),
                                    ),
                                    start=True,
                                    space=self._space,
                                    out=out,
                                )

                                return 404



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Тестирование разработанных модулей
"""

import os
import sys

PATH_TO_SOURCE = os.path.abspath(os.path.dirname(__file__))
PATH_TO_ROOT = os.path.join(PATH_TO_SOURCE, "..", "..")

sys.path.insert(0, os.path.abspath(PATH_TO_ROOT))

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
# Подавление Warning
import warnings

for warn in [UserWarning, FutureWarning]:
    warnings.filterwarnings("ignore", category=warn)

from dataclasses import dataclass  # Класс данных

import logging  # Логирование Функции создающие итераторы для эффективного цикла

# Типы данных
from typing import Dict, Union, Any
from types import ModuleType

# Персональные
import openav  # Библиотека в целом
from openav.modules.trml.shell import Shell  # Работа с Shell
from openav.modules.lab.build import Run  # Сборка библиотеки
from openav import rsrs  # Ресурсы библиотеки

from openav.modules.core.logging import ARG_PATH_TO_LOGS
from openav.modules.lab.audio import (
    TYPES_ENCODE,
    PRESETS_CRF_ENCODE,
    SR_INPUT_TYPES,
    SAMPLING_RATE_VAD,
    VOSK_SUPPORTED_LANGUAGES,
    VOSK_SUPPORTED_DICTS,
)


# ######################################################################################################################
# Сообщения
# ######################################################################################################################
@dataclass
class MessageTesting(Run):
    """Класс для сообщений"""

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._description: str = self._("Тестирование разработанных модулей")
        self._description_time: str = "{}" * 2 + self._description + self._em + "{}"

        self._check_config_file_valid = self._("Проверка данных на валидность") + self._em


# ######################################################################################################################
# Выполняем только в том случае, если файл запущен сам по себе
# ######################################################################################################################
@dataclass
class RunTesting(MessageTesting):
    """Класс для тестирования разработанных модулей"""

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        #  Регистратор логирования с указанным именем
        self._logger_run_vosk_sr: logging.Logger = logging.getLogger(__class__.__name__)
        """Загрузка и проверка конфигурационного файла

        Args:
            resources (ModuleType): Модуль с ресурсами
            config (str): Конфигурационный файл
            out (bool): Печатать процесс выполнения

        Returns:
             bool: **True** если конфигурационный файл загружен и валиден, в обратном случае **False**
        """

        # Проверка аргументов
        if not isinstance(resources, ModuleType) or type(config) is not str or not config or type(out) is not bool:
            try:
                raise TypeError
            except TypeError:
                self.inv_args(__class__.__name__, self._load_config_yaml.__name__, out=out)
                return False

        # Конфигурационный файл передан
        if self._args["config"] is not None:
            config_yaml = self.load_yaml(self._args["config"], False, out)  # Загрузка YAML файла
        else:
            config_yaml = self.load_yaml_resources(resources, config, out)  # Загрузка YAML файла из ресурсов модуля

        # Конфигурационный файл не загружен
        if not config_yaml:
            return False

        # Проверка конфигурационного файла на валидность
        res_valid_yaml_config = self._valid_yaml_config(config_yaml, out)

        # Конфигурационный файл не валидный
        if res_valid_yaml_config is False:
            return False

        # Проход по всем разделам конфигурационного файла
        for k, v in config_yaml.items():
            self._args[k] = v  # Добавление значения из конфигурационного файла в словарь аргументов командной строки

        return True

    # ------------------------------------------------------------------------------------------------------------------
    #  Внешние методы
    # ------------------------------------------------------------------------------------------------------------------

    def run(self, metadata: ModuleType = openav, resources: ModuleType = rsrs, out: bool = True) -> bool:
        """Запуск тестирования разработанных модулей

        Args:
            metadata (ModuleType): Модуль из которого необходимо извлечь информацию
            resources (ModuleType): Модуль с ресурсами
            out (bool): Печатать процесс выполнения

        Returns:
             bool: **True** если тестирование разработанных модулей произведено успешно,
                   в обратном случае **False**
        """

        # Проверка аргументов
        if not isinstance(metadata, ModuleType) or not isinstance(resources, ModuleType) or type(out) is not bool:
            try:
                raise TypeError
            except TypeError:
                self.inv_args(__class__.__name__, self.run.__name__, out=out)
                return False

        self._args = self._build_args(self._description)  # Построение аргументов командной строки
        if len(self._args) == 0:
            return False

        # Очистка консоли перед выполнением
        if self.clear_shell(cls=self._args["no_clear_shell"], out=True) is False:
            return False

        # Вывод сообщения
        if out is True:
            # Приветствие
            Shell.add_line()  # Добавление линии во весь экран
            print(self._description_time.format(self.text_bold, self.color_blue, self.text_end))
            self._logger_run_vosk_sr.info(self._description)
            Shell.add_line()  # Добавление линии во весь экран

        # Загрузка и проверка конфигурационного файла
        if self._load_config_yaml(resources, out=out) is False:
            return False

        # Вывод сообщения
        if out is True:
            Shell.add_line()  # Добавление линии во весь экран

        # Информация об библиотеке
        if self._args["hide_metadata"] is False and out is True:
            self.message_metadata_info(out=out)
            Shell.add_line()  # Добавление линии во весь экран

        # Версии установленных библиотек
        if self._args["hide_libs_vers"] is False and out is True:
            self.libs_vers(out=out)
            Shell.add_line()  # Добавление линии во весь экран

        # Проверка модуля загрузки данных
        self.download_data(out=out)

        # Проверка модуля детектирования речевой активности
        self.vosk_sr(
            depth=self._args["depth"],  # Глубина иерархии для получения данных
            type_encode=self._args["type_encode"],  # Тип кодирования
            crf_value=self._args["crf_value"],  # Качество кодирования
            presets_crf_encode=self._args["presets_crf_encode"],  # Скорость кодирования и сжатия
            new_name=self._args["folder_name_unzip"],  # Имя директории для разархивирования
            # Внутренний левый отступ для итоговых речевых фрагментов
            speech_left_pad_ms=self._args["speech_left_pad_ms"],
            # Внутренний правый отступ для итоговых речевых фрагментов
            speech_right_pad_ms=self._args["speech_right_pad_ms"],
            force_reload=self._args["force_reload"],  # Принудительная загрузка модели из сети
            # Очистка директории для сохранения фрагментов аудиовизуального сигнала
            clear_dirvosk_sr=self._args["clear_dirvosk_sr"],
            out=out,
        )

        # Проверка модуля предобработки речевых аудиоданных
        self.preprocessing_audio(
            sample_rate=self._args["sample_rate"],  # Частота дискретизации аудиосигнала
            n_fft=self._args["n_fft"],  # Размер параметра FFT
            hop_length=self._args["hop_length"],  # Длина перехода между окнами STFT
            n_mels=self._args["n_mels"],  # Количество фильтроблоков mel
            power=self._args["power"],  # Показатель степени магнитудной спектрограммы
            pad_mode=self._args["pad_mode"],  # Управление оступами
            center=self._args["center"],  # Включение установки отступов с обеих сторон относительно центра
            # Очистка директории для сохранения фрагментов аудиовизуального сигнала
            clear_dirvosk_sr=self._args["clear_dirvosk_sr"],
            out=out,
        )

        # Проверка модуля предобработки речевых видеоданных
        self.preprocessing_video(
            depth=self._args["depth"],  # Глубина иерархии для получения данных
            width=self._args["width"],  # Ширина кадра с найденной областью губ
            height=self._args["height"],  # Высота кадра с найденной областью губ
            color_mode=self._args["color_mode"],  # Цветовая гамма конечного изображения
            # Очистка директории для сохранения фрагментов аудиовизуального сигнала
            clear_dirvosk_sr=self._args["clear_dirvosk_sr"],
            out=out,
        )

        return True


def main():
    # Запуск тестирования разработанных модулей
    vad = RunTesting(lang="ru", path_to_logs="./openav/logs")
    vad.run(out=True)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Логирование
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
# Подавление Warning
import warnings

for warn in [UserWarning, FutureWarning]:
    warnings.filterwarnings("ignore", category=warn)

import os  # Взаимодействие с файловой системой
import sys  # Доступ к некоторым переменным и функциям Python
import re  # Регулярные выражения

from dataclasses import dataclass, field  # Класс данных

import logging  # Логирование

# Типы данных
from typing import List, Optional

# Персональные
from openav import __title__

# ######################################################################################################################
# Константы
# ######################################################################################################################
ARG_PATH_TO_LOGS = "--path_to_logs"  # Аргумент в парсере командной строки
PATH_TO_LOGS: str = os.path.normpath("./logs")  # Путь к директории для сохранения LOG файлов
NAME_LOG: str = __title__ + ".log"  # Имя LOG файла


# ######################################################################################################################
# Логирование
# ######################################################################################################################
@dataclass
class Logging:
    """Класс для логирования

    Args:
        path_to_logs (str): Путь к директории для сохранения LOG файлов
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    path_to_logs: str
    __path_to_logs: str = field(default=PATH_TO_LOGS, init=False, repr=False)
    """Путь к директории для сохранения LOG файлов

    .. note::
        private (приватный аргумент)
    """

    def __post_init__(self):
        self.path_to_logs = self.__get_arg()  # Установка пути к директории для сохранения LOG файлов

        self._logger_handler: Optional[logging.NullHandler] = None  # Обработчик логирования
        self.__logger: bool = self.__create_logging()  # Создание регистратора и обработчика для логирования

        self.logger_gui: bool = False  # Логирование GUI
        self._logger_messages_gui: List[str] = []  # Сообщения для логирования в режиме GUI

    # ------------------------------------------------------------------------------------------------------------------
    # Свойства
    # ------------------------------------------------------------------------------------------------------------------

    @property
    def path_to_logs(self) -> str:
        """Получение/установка пути к директории для сохранения LOG файлов

        Args:
            (str): Путь

        Returns:
            str: Путь
        """

        return self.__path_to_logs

    @path_to_logs.setter
    def path_to_logs(self, path: str):
        """Установка пути к директории для сохранения LOG файлов"""

        try:
            # Проверка аргументов
            if type(path) is not str or not path:
                raise TypeError
        except TypeError:
            pass
        else:
            self.__path_to_logs = os.path.normpath(path.strip())

    @property
    def check_create_logger(self) -> bool:
        """Получение создания регистратора и обработчика для логирования

        Returns:
            bool: **True** если регистратор и обработчик созданы, в обратном случае **False**
        """

        return self.__logger

    @property
    def logger_gui(self) -> bool:
        """Получение/установка логирования GUI

        Args:
            (bool): **True** если запущен GUI, в обратном случае **False**

        Returns:
            bool: **True** если запущен GUI, в обратном случае **False**
        """

        return self.__logger_gui

    @logger_gui.setter
    def logger_gui(self, run_gui: bool):
        """Установка логирования GUI"""

        try:
            # Проверка аргументов
            if type(run_gui) is not bool:
                raise TypeError
        except TypeError:
            pass
        else:
            self.__logger_gui = run_gui

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (приватные)
    # ------------------------------------------------------------------------------------------------------------------

    @staticmethod
    def __get_arg() -> str:
        """Получение аргумента в парсере командной строки (путь к директории для сохранения LOG файлов)

        .. note::
            private (приватный метод)

        Returns:
            str: Путь к директории для сохранения LOG файлов
        """

        for cnt, arg in enumerate(sys.argv):
            if arg == ARG_PATH_TO_LOGS:
                try:
                    return sys.argv[cnt + 1]
                except IndexError:
                    return ""
        return ""

    def __create_logging(self) -> bool:
        """Создание регистратора и обработчика для логирования

        .. note::
            private (приватный метод)

        Returns:
            bool: **True** если регистратор и обработчик созданы, в обратном случае **False**
        """

        if not os.path.exists(self.path_to_logs):
            try:
                os.makedirs(self.path_to_logs)
            except Exception:
                return False

        try:
            # Создание регистратора
            logging.basicConfig(
                filename=os.path.join(self.path_to_logs, NAME_LOG),
                encoding="utf-8",
                filemode="w",
                format="%(asctime)s.%(msecs)03d - %(name)s:%(funcName)s:%(lineno)d - %(levelname)s - %(message)s",
                datefmt="%Y-%m-%d %H:%M:%S",
                level=logging.DEBUG,
            )
        except FileNotFoundError:
            return False
        except Exception:
            return False
        else:
            # Создание обработчика
            self._logger_handler = logging.NullHandler()

            self._logger_handler.setLevel(logging.DEBUG)

            logging.getLogger("requests").setLevel(logging.CRITICAL)
            logging.getLogger("urllib3").setLevel(logging.CRITICAL)
            logging.getLogger("").addHandler(self._logger_handler)

            return True

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (защищенные)
    # ------------------------------------------------------------------------------------------------------------------

    def _get_logger_messages_gui(self) -> List[str]:
        """Получение сообщений для логирования в режиме GUI

        .. note::
            protected (защищенный метод)

        Returns:
            List[str]: Сообщения
        """

        return self._logger_messages_gui

    def _add_logger_messages_gui(self, message: str, last: bool = False) -> None:
        """Добавление сообщения для логирования в режиме GUI

        .. note::
            protected (защищенный метод)

        Args:
            message (str): Сообщение
            last (bool): Замена последнего сообщения

        Returns:
            None
        """

        if last is True:
            try:
                self._logger_messages_gui[-1] = message
            except Exception:
                pass
            else:
                return None

        self._logger_messages_gui.append(message)

    # ------------------------------------------------------------------------------------------------------------------
    # Внешние методы
    # ------------------------------------------------------------------------------------------------------------------

    @staticmethod
    def re_inv_chars(path: str) -> str:
        """Удаление недопустимых символов из пути

        Args:
            path (str): Путь

        Returns:
            str: Путь
        """

        # Проверка аргументов
        if type(path) is not str or not path:
            return ""

        return re.sub('[\\/:"*?<>|]+', "", path)
